struct DropScopeToken;Expand description
A token to ensuring that each drop scope is popped at most once, thanks to the compiler that checks moves.
Implementations
Sourceimpl DropScopeToken
impl DropScopeToken
fn pop_and_drop<'db>( self, ctx: &mut MirLowerCtx<'_, 'db>, current: BasicBlockId, span: MirSpan, ) -> BasicBlockId
Sourcefn pop_assume_dropped(self, ctx: &mut MirLowerCtx<'_, '_>)
fn pop_assume_dropped(self, ctx: &mut MirLowerCtx<'_, '_>)
It is useful when we want a drop scope is syntactically closed, but we don’t want to execute any drop code. Either when the control flow is diverging (so drop code doesn’t reached) or when drop is handled for us (for example a block that ended with a return statement. Return will drop everything, so the block shouldn’t do anything)